! Dean Townsley 2010
!
! initialize helium layers on carbon with base at bposition in HSE
!     dens_he_c is the density at the He-C layer interface (base of accreted layer)
subroutine envelope_hse(dens, temp, he, cz_base_position,&
                     dens_he_c, temp_base_cz, temp_under, y_he_under_cz, &
                     x_c12_cz, x_o16_cz, x_c12_under, x_o16_under, bposition, x1, dx, grav, nfill)
  use hse_interface, ONLY: sim_hse_step
  use Flame_interface, ONLY : Flame_getProfile, Flame_getWidth
  use Simulation_data, ONLY: HSE_FORWARD, HSE_BACKWARD, HSE_SETTEMP, HSE_CONSTENTR
  implicit none
  real, dimension(:), intent(OUT)  :: dens, temp, he
  real, intent(OUT)                :: cz_base_position
  real, intent(IN)                 :: dens_he_c, temp_base_cz, temp_under, y_he_under_cz
  real, intent(IN)                 :: x_c12_cz, x_o16_cz, x_c12_under, x_o16_under, bposition, x1, dx, grav
  integer, intent(IN)              :: nfill
  ! x1 is (center) coordinate of first element in return arrays, dx is spacing and nfill
  ! is the number of cells that this function should fill
  ! also an isothermal helium layer of extent y_he_under_cz is placed under the convection zone

  real :: x, y_he_sum
  integer :: i, refi
  real,allocatable,dimension(:) :: ye, sumy
  logical :: in_conv_zone


!=================================================================

  allocate(ye(nfill))
  allocate(sumy(nfill))
  ! set up profile first without worrying about HSE
  do i = 1, nfill
     x = x1+dx*(i-1)
     temp(i) = temp_under
     if (x >= bposition) then
        ! in helium layer
        ye(i) = 0.5
        sumy(i) = 0.25*(1-x_c12_under-x_o16_under)+x_c12_under/12.0+x_o16_under/16.0
        he(i) = 1.0 - x_c12_under - x_o16_under
     else
        ! in "carbon" layer
        ye(i) = 0.5
        sumy(i) = 1.0/24.0
        he(i) = 0.0
     endif
  enddo
     
  ! the reference point is the first cell above the interface
  refi = max(1,int( (bposition-x1)/dx + 2 ) )
  dens(refi) = dens_he_c
  !! first go up from reference point
  ! guess value
  dens(refi+1) = dens(refi)
  y_he_sum = dens(refi)*dx
  ! determine whether there is as cold layer on the grid or not
  ! if the cold layer requested is smaller than one grid cell can provide
  ! we just do a convection zone
  if (y_he_under_cz < y_he_sum) then
     temp(refi) = temp_base_cz
     he(i) = 1.0 - x_c12_cz - x_o16_cz
     sumy(i) = he(i)*0.25 + x_c12_cz/12.0 + x_o16_cz/16.0
     in_conv_zone=.true.
     ! save base position as the lower edge of this cell
     cz_base_position = x1 + (refi-0.5)*dx
     ! first step is first order
     call sim_hse_step(dens,temp,ye,sumy,refi+1,grav,dx,HSE_FORWARD,1,HSE_CONSTENTR)
  else
     in_conv_zone = .false.
     ! first step is first order
     call sim_hse_step(dens,temp,ye,sumy,refi+1,grav,dx,HSE_FORWARD,1,HSE_SETTEMP)
  endif
  do i = refi + 2, nfill
     dens(i) = dens(i-1)
     y_he_sum = y_he_sum + dens(i-1)*dx
     if (dens(i) < 100.0) then
        ! short circuit for outer fluff
        ! propagate temperature also from outermost HSE zone
        he(i) = 1.0 - x_c12_cz - x_o16_cz
        sumy(i) = he(i)*0.25 + x_c12_cz/12.0 + x_o16_cz/16.0
        temp(i) = temp(i-1)
     else if (.not. in_conv_zone) then
        ! const temperature until we reach extent of he layer beneath conv. zone
        if (y_he_sum > y_he_under_cz) then
           temp(i) = temp_base_cz
           he(i) = 1.0 - x_c12_cz - x_o16_cz
           sumy(i) = he(i)*0.25 + x_c12_cz/12.0 + x_o16_cz/16.0
           in_conv_zone=.true.
           ! save base position as the lower edge of this cell
           cz_base_position = x1 + (i-0.5)*dx
        endif
        call sim_hse_step(dens,temp,ye,sumy,i,grav,dx,HSE_FORWARD,2,HSE_SETTEMP)
     else
        ! iso entropy (convective) the rest of the way up to fluff
        he(i) = 1.0 - x_c12_cz - x_o16_cz
        sumy(i) = he(i)*0.25 + x_c12_cz/12.0 + x_o16_cz/16.0
        call sim_hse_step(dens,temp,ye,sumy,i,grav,dx,HSE_FORWARD,2,HSE_CONSTENTR)
     endif
  enddo
  !! now go down from reference point
  do i = refi-1, 1, -1
     dens(i) = dens(i+1)
     call sim_hse_step(dens,temp,ye,sumy,i,grav,dx,HSE_BACKWARD,2,HSE_SETTEMP)
  enddo

  deallocate(ye)
  deallocate(sumy)

  return

end subroutine
